import gtk
import gtk.glade
import gobject

from newtask import PyAtCronNewTask
from about import PyAtCronAbout

# The main window's class.
class PyAtCronMainwin:

	def __init__ (self):
		# The Id of the task currently highlighted (if any).
		self.taskId = None

		# The GtkTreePath of the last highlighted inactive task. It's used to switch between the inactive/highlighted colors in the task list.
		self.lastPath = None

		# We load the widgets and the callbacks from the glade file.
		self.widgetTree = gtk.glade.XML ("glade/mainwin.glade")
		dic = {
						"on_mainwin_delete_event" : self.deleteEvent,
						"on_mainwin_destroy_event" : self.destroyEvent,
						"on_tasklist_button_press_event" : self.buttonPressEvent,
						"on_mainmenu_save_activate" : self.saveActivate,
						"on_mainmenu_quit_activate" : self.quitActivate,
						"on_mainmenu_new_activate" : self.newActivate,
						"on_mainmenu_properties_activate" : self.propertiesActivate,
						"on_mainmenu_activate_activate" : self.activateActivate,
						"on_mainmenu_deactivate_activate" : self.deactivateActivate,
						"on_mainmenu_delete_activate" : self.deleteActivate,
						"on_mainmenu_about_activate" : self.aboutActivate,
						"on_popup_properties_activate" : self.propertiesActivate,
						"on_popup_activate_activate" : self.activateActivate,
						"on_popup_deactivate_activate" : self.deactivateActivate,
						"on_popup_delete_activate" : self.deleteActivate,
						"on_toolbar_new_clicked" : self.newClicked,
						"on_toolbar_properties_clicked" : self.propertiesClicked,
						"on_toolbar_activate_clicked" : self.activateClicked,
						"on_toolbar_deactivate_clicked" : self.deactivateClicked,
						"on_toolbar_delete_clicked" : self.deleteClicked,
						"on_tasklist_row_activated" : self.rowActivated
					}
		self.widgetTree.signal_autoconnect (dic)

		# When the selection changes, we call self.selectionChanged (). This can't be handled directly by glade if not by using the on_cursor_changed callback, which doesn't work well.
		self.widgetTree.get_widget ("tasklist").get_selection().connect("changed", self.selectionChanged)

		# It's the base widget.
		self.win = self.widgetTree.get_widget ("mainwin")

		# This is used to get the "highlighted" color from the GTK theme.
		self.highlightedColor = self.getThemeColor (3)

		# This is used to get the "insensitive" color from the GTK theme.
		self.insensitiveColor = self.getThemeColor (4)

		# This is used to get the main background color from the GTK theme.
		self.backgroundColor = self.getThemeColor (8)

		# We build the task list.
		self.updateTaskList ()

		# We get the current highlighted task's Id.
		self.updateCurrentTask ()

	# Gets the colors from the GTK theme
	def getThemeColor (self, num):
		if (num < 5):
			redhex = hex (self.win.rc_get_style ().fg [num].red)
		else:
			redhex = hex (self.win.rc_get_style ().bg [num - 5].red)
		redsplit = redhex.split ('0x')
		redstr = redsplit [1]
		
		if (num < 5):
			greenhex = hex (self.win.rc_get_style ().fg [num].green)
		else:
			greenhex = hex (self.win.rc_get_style ().bg [num - 5].green)
		greensplit = greenhex.split ('0x')
		greenstr = greensplit [1]
		
		if (num < 5):
			bluehex = hex (self.win.rc_get_style ().fg [num].blue)
		else:
			bluehex = hex (self.win.rc_get_style ().bg [num - 5].blue)
		bluesplit = bluehex.split ('0x')
		bluestr = bluesplit [1]
		
		color = "#" + redstr + greenstr + bluestr

		return color

	def deleteEvent (self, obj, event):
		self.quit ()

	def destroyEvent (self, obj, event):
		self.quit ()

	def buttonPressEvent (self, obj, event):
		# If the right-mouse button is pressed, get the current selection.
		if (event.button == 3):
			taskList = self.widgetTree.get_widget ("tasklist")
			iter = taskList.get_selection ().get_selected () [1]

			# If there is a selection, check that the click occured inside the selected row.
			if (iter != None ):
				model = taskList.get_model () 
				path = model.get_path (iter)
				column = taskList.get_column (0)
				cell_area = taskList.get_cell_area (path, column)
				# If it's the case, setup and display the popup menu
				if (event.y >= cell_area.y and event.y <= cell_area.y + cell_area.height):
					activable = model.get_value (iter, 1)

					self.widgetTree.get_widget ("popup_properties").set_sensitive (1)
					self.widgetTree.get_widget ("popup_activate").set_sensitive (activable)
					self.widgetTree.get_widget ("popup_deactivate").set_sensitive (1 - activable)
					self.widgetTree.get_widget ("popup_delete").set_sensitive (1)


					self.widgetTree.get_widget ("popup").popup (None, None, None, event.button, event.time)

	def saveActivate (self, obj):
		self.save ()

	def quitActivate (self, obj):
		self.quit ()

	def newActivate (self, obj):
		self.createTask ()

	def propertiesActivate (self, obj):
		self.editTask ()

	def activateActivate (self, obj):
		self.activateTask ()

	def deactivateActivate (self, obj):
		self.deactivateTask ()

	def deleteActivate (self, obj):
		self.confirmDeleteTask ()

	def aboutActivate (self, obj):
		self.about ()

	def newClicked (self, obj):
		self.createTask ()

	def propertiesClicked (self, obj):
		self.editTask ()

	def activateClicked (self, obj):
		self.activateTask ()

	def deactivateClicked (self, obj):
		self.deactivateTask ()

	def deleteClicked (self, obj):
		self.confirmDeleteTask ()

	def rowActivated (self, obj, path, column):
		self.editTask ()

	def selectionChanged (self, obj):
		self.updateCurrentTask ()

	def confirmDeleteResponse (self, obj, response):
		# If deletion is confirmed, delete the task.
		if (response == -8):
			self.deleteTask ()

		# Free the dialog.
		obj.destroy ()

	# Writes the current configuration.
	def save (self):
		print "TODO: save"

	# Unreferences the main window's widgets, and exits.
	def quit (self):
		self.save ()
		gtk.main_quit ()

	# Gets the current highlighted task's Id.
	def updateCurrentTask (self):
		taskList = self.widgetTree.get_widget ("tasklist")
		iter = taskList.get_selection ().get_selected () [1]
		model = taskList.get_model ()

		# If we were on an inactive task, restore its color to "insensitive".
		if (self.lastPath != None):
			model.set_value (model.get_iter (self.lastPath), 4, self.insensitiveColor)
			self.lastPath = None

		# If no row is selected, the "properties", "activate", "deactivate" and "delete" toolbar icons/mainmenu entries are insensitive, else they are sensitive.
		if (iter == None):
			self.widgetTree.get_widget ("toolbar_properties").set_sensitive (0)
			self.widgetTree.get_widget ("toolbar_activate").set_sensitive (0)
			self.widgetTree.get_widget ("toolbar_deactivate").set_sensitive (0)
			self.widgetTree.get_widget ("toolbar_delete").set_sensitive (0)
			self.widgetTree.get_widget ("mainmenu_properties").set_sensitive (0)
			self.widgetTree.get_widget ("mainmenu_activate").set_sensitive (0)
			self.widgetTree.get_widget ("mainmenu_deactivate").set_sensitive (0)
			self.widgetTree.get_widget ("mainmenu_delete").set_sensitive (0)
		else:
			# If the task is inactive, the "deactivate" icon toolbar is insensitive, the "activate" icon is sensitive and vice-versa.
			activable = model.get_value (iter, 1)

			self.widgetTree.get_widget ("toolbar_properties").set_sensitive (1)
			self.widgetTree.get_widget ("toolbar_activate").set_sensitive (activable)
			self.widgetTree.get_widget ("toolbar_deactivate").set_sensitive (1 - activable)
			self.widgetTree.get_widget ("toolbar_delete").set_sensitive (1)
			self.widgetTree.get_widget ("mainmenu_properties").set_sensitive (1)
			self.widgetTree.get_widget ("mainmenu_activate").set_sensitive (activable)
			self.widgetTree.get_widget ("mainmenu_deactivate").set_sensitive (1 - activable)
			self.widgetTree.get_widget ("mainmenu_delete").set_sensitive (1)

			# Here, we get the task's Id.
			self.taskId = model.get_value (iter, 0)

			if (activable == 1):
				# We save the task's path for next time.
				path = taskList.get_cursor () [0]
				self.lastPath = taskList.get_cursor () [0]
				# We set the inactive task's color to "highlighted".
				model.set_value (model.get_iter (path), 4, self.highlightedColor)

	# Launches the new task assistant and refreshes the task list if a new task was created.
	def createTask (self):
		assistantObject = PyAtCronNewTask (self)
		self.updateTaskList ()

	# Launches the properties dialog and refreshes the task list if changes were made.
	def editTask (self):
		print "TODO: edit task #" + str (self.taskId)
		self.updateTaskList ()

	# Makes the current task active and refreshes the task list.
	def activateTask (self):
		print "TODO: activate task #" + str (self.taskId)
		self.updateTaskList ()

	# Makes the current task inactive and refreshes the task list.
	def deactivateTask (self):
		print "TODO: deactivate task #" + str (self.taskId)
		self.updateTaskList ()

	# Show a confirmation dialog before deleting the task.
	def confirmDeleteTask (self):
		# Load the dialog.
		xml = gtk.glade.XML ("glade/confirm_delete.glade")
		dic = {
			      "on_confirm_delete_response" : self.confirmDeleteResponse,
					}
		xml.signal_autoconnect (dic)
		confirm = xml.get_widget ("confirm_delete")

		# Center the dialog on the main window.
		confirm.set_transient_for (self.win)
		confirm.set_position (gtk.WIN_POS_CENTER_ON_PARENT)

		# Display the dialog.
		confirm.show ()

	# Deletes the current task and refreshes the task list.
	def deleteTask (self):
		print "TODO: delete task #" + str (self.taskId)
		self.updateTaskList ()

	# Launches the "about" dialog.
	def about (self):
		aboutObject = PyAtCronAbout (self.win)

	# Builds or refreshes the task list.
	def updateTaskList (self):
		taskList = self.widgetTree.get_widget ("tasklist")

		# If the task list has not already been built, ceate it. Else, get its current model, get the position of the cursor from it and clear the current model.
		if (taskList.get_model () == None):
			model = gtk.ListStore (gobject.TYPE_INT, gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
		else:
			model = taskList.get_model ()
			cursor = taskList.get_cursor ()
			model.clear ()

		# Add each task.
		model.set (model.append (), 0, 2, 1, False, 2, "Save all my HOME folder", 3, "12/01/03 4:00 AM", 4, self.insensitiveColor)
		model.set (model.append (), 0, 4, 1, True, 2, "Wake me up on the morning", 3, "Tomorrow 7:00 AM", 4, self.insensitiveColor)
		model.set (model.append (), 0, 6, 1, False, 2, "Start downloading CD1", 3, "Today 9:00 PM", 4, self.insensitiveColor)
		model.set (model.append (), 0, 8, 1, False, 2, "Start downloading CD2", 3, "Today 11:00 PM", 4, self.insensitiveColor)
		model.set (model.append (), 0, 10, 1, False, 2, "Start downloading CD3", 3, "Tomorrow 2:00 AM", 4, self.insensitiveColor)

		# If we are building the task list, associate the model to the task list and create the headers. Else, restore the cursor's position.
		if (taskList.get_model () == None):
			taskList.set_model (model)

			textRenderer = gtk.CellRendererText ()
			textColumn = gtk.TreeViewColumn ("Name", textRenderer, text=2, foreground_set=1, foreground=4)
			taskList.append_column (textColumn)

			textRenderer = gtk.CellRendererText ()
			textColumn = gtk.TreeViewColumn ("Next run time", textRenderer, text=3, foreground_set=1, foreground=4)
			taskList.append_column (textColumn)
		else:
			if (cursor [0] != None):
				taskList.set_cursor (cursor [0])
