import gtk
import gtk.glade
import gobject

# The new task assistant's class.
class PyAtCronNewTask:

	def __init__ (self, parent):
		# This the assistant's caller
		self.parent = parent

		# This is the current page.
		self.currentPage = 1

		# This is the last page.
		self.lastPage = 1

		# This is the previously displayed page.
		self.oldPage = None

		# This is the text displayed on page 1.
		self.taskTypeSelectionText = "Use the list on the left to choose the type of task you would like to add."

		# This is the Id of the selected task type (if any).
		self.taskTypeId = None

		# We load the widgets and the callbacks from the glade file.
		self.widgetTree = gtk.glade.XML ("glade/newtask.glade")
		dic = {
						"on_newtask_delete_event" : self.deleteEvent,
						"on_cancel_button_clicked" : self.cancelClicked,
						"on_back_button_clicked" : self.backClicked,
						"on_forward_button_clicked" : self.forwardClicked
					}
		self.widgetTree.signal_autoconnect (dic)

		# It's the base widget.
		self.win = self.widgetTree.get_widget ("newtask")

		# Center the window on the parent.
		self.win.set_transient_for (self.parent.win)
		self.win.set_position (gtk.WIN_POS_CENTER_ON_PARENT)

		# Sets the color of the bars.
		frameBackground = gtk.gdk.color_parse (self.parent.backgroundColor)
		mainBackground = gtk.gdk.color_parse (self.parent.highlightedColor)

		self.widgetTree.get_widget ("titlebar").modify_bg (gtk.STATE_NORMAL, frameBackground)
		self.widgetTree.get_widget ("topbar").modify_bg (gtk.STATE_NORMAL, frameBackground)
		self.widgetTree.get_widget ("bottombar").modify_bg (gtk.STATE_NORMAL, frameBackground)
		self.widgetTree.get_widget ("leftbar").modify_bg (gtk.STATE_NORMAL, frameBackground)
		self.widgetTree.get_widget ("rightbar").modify_bg (gtk.STATE_NORMAL, frameBackground)
		self.widgetTree.get_widget ("mainbar").modify_bg (gtk.STATE_NORMAL, mainBackground)
		self.widgetTree.get_widget ("widgetbar").modify_bg (gtk.STATE_NORMAL, mainBackground)

		# The first page is the task type selection page.
		pageWidget1 = gtk.HBox ()

		frame = gtk.Viewport ()

		taskTypeList = gtk.TreeView ()
		taskTypeList.set_headers_visible (False)

		# When the selection changes, we call self.taskTypeChanged ().
		taskTypeList.get_selection().connect("changed", self.taskTypeChanged)

		model = gtk.ListStore (gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_STRING)

		# Here comes the list of task types.
		model.set (model.append (), 0, 1, 1, "Backup", 2, "Saves and compresses a given folder.")
		model.set (model.append (), 0, 2, 1, "Download", 2, "Retrieves one or more file(s) from a remote computer.")
		model.set (model.append (), 0, 3, 1, "Command", 2, "Executes a given shell command.")

		taskTypeList.set_model (model)

		# We define the header, but we don't display it.
		textRenderer = gtk.CellRendererText ()
		textColumn = gtk.TreeViewColumn ("Type of task", textRenderer, text=1)
		taskTypeList.append_column (textColumn)

		# This is the place where we display comments about of each task type
		comment = gtk.Label (self.taskTypeSelectionText)
		comment.set_padding (4, 4)
		comment.set_alignment (0.0, 0.0)
		comment.set_line_wrap (gtk.TRUE)
		comment.set_size_request (340, -1)

		# The page is assembled and displayed.
		frame.add (taskTypeList)
		taskTypeList.show ()
		pageWidget1.add (frame)
		pageWidget1.set_child_packing (frame, False, False, 0, 0)
		frame.show ()
		pageWidget1.add (comment)
		comment.show ()

		# This page is the properties page. It depends on the task type.
		pageWidget2 = gtk.Label ("Configure the task")
		pageWidget2.set_alignment (0.0, 0.0)

		pageWidget3 = gtk.Label ("Set the task's schedule")
		pageWidget3.set_alignment (0.0, 0.0)

		self.pages = {
		               1 : {
                         "title" : "Choose the type of task",
                         "widget" : pageWidget1
                       },
		               2 : {
                         "title" : "Configure the task",
                         "widget" : pageWidget2
                       },
		               3 : {
                         "title" : "Set the task's schedule",
                         "widget" : pageWidget3
                       }
		             }
		self.lastPage = 3

		# Displays the assistant.
		self.win.show ()

		# Update the currently viewed page
		self.updatePage ()

	def deleteEvent (self, obj, event):
		self.quit ()

	def backClicked (self, obj):
		self.goBack ()

	def forwardClicked (self, obj):
		self.goForward ()

	def cancelClicked (self, obj):
		self.quit ()

	def goBack (self):
		# If we're not on the first page, go back
		if (self.currentPage > 1):
			self.currentPage = self.currentPage - 1
			self.updatePage ()

	def goForward (self):
		# If we're not on the last page, go forward. Else, finish with the assistant.
		if (self.currentPage < self.lastPage):
			self.currentPage = self.currentPage + 1
			self.updatePage ()
		else:
			print "TODO: finish"
			self.quit ()

	def updatePage (self):
		# If we are of the first page, make the back button insensitive and prevent from going forward without selecting a task type.
		self.widgetTree.get_widget ("back_button").set_sensitive (self.currentPage != 1)
		self.widgetTree.get_widget ("forward_button").set_sensitive ((self.taskTypeId != None))

		# If we are of the last page, the button's text to "OK". Else, it's "Forward".
		self.widgetTree.get_widget ("forward_button").set_use_stock (True)
		if (self.currentPage == self.lastPage):
			self.widgetTree.get_widget ("forward_button").set_label (eval('gtk.STOCK_OK'))
		else:
			self.widgetTree.get_widget ("forward_button").set_label (eval('gtk.STOCK_GO_FORWARD'))

		# If we currently have a page displayed, remove it.
		if (self.oldPage != None):
			self.widgetTree.get_widget ("widgetbar").remove (self.oldPage)

		# Set the assistant's title.
		title = '<span size="xx-large" color="' + self.parent.highlightedColor + '"><b>' + self.pages [self.currentPage]["title"] + '</b></span>'
		self.widgetTree.get_widget ("title").set_markup (title)

		# Add the current page to the dialog (with a 4 pixels padding), and display it.
		page = self.pages [self.currentPage]["widget"]
		self.widgetTree.get_widget ("widgetbar").add (page)
		page.show ()

		# Save this page, to remove it before displayed the next page.
		self.oldPage = page

	# Display the comment matching the selected type of task, and activate/deactivate the forward button.
	def taskTypeChanged (self, obj):
		taskTypeList = self.widgetTree.get_widget ("widgetbar").get_child ().get_children () [0].get_child ()

		commentWidget = self.widgetTree.get_widget ("widgetbar").get_child ().get_children () [1]

		iter = obj.get_selected () [1]

		# Display the right comment, or the default text.
		if (iter == None):
			comment = self.taskTypeSelectionText
			# No task selected
			self.taskTypeId = None
		else:
			# Set the taskTypeId
			self.taskTypeId = taskTypeList.get_model ().get_value (iter, 0)
			comment = taskTypeList.get_model ().get_value (iter, 2)
		commentWidget.set_text (comment)

		# You can't go forward without selectiong a task type
		self.widgetTree.get_widget ("forward_button").set_sensitive ((self.taskTypeId != None))

	# Unreferences the main window's widgets.
	def quit (self):
		self.win.destroy ()
